/* Copyright IBM Corp. 2008  All Rights Reserved.                    */
dojo.provide("com.ibm.mashups.livetext.service");

/**
   The livetext (formally known as semantic tag) service API. The implementation class of this API provides the livetext parsing services 
   so that tags on a page can be recognized and processed according to the tagservice.entries.cfg
   configuration file. An instance of this API implementation class should be created after a page is fully loaded. 
   Once the instance is created, the init method should be called so that the tags will be parsed. Each tag processor 
   will be invoked to process the tags in order of the tags defined in the tagservices.entries.cfg file. There should be no more
   than 1 instance of this class on each page. 
   	
   The implementation of this API should subscribe to one of the following two event topics.<p/>
   
   &nbsp;&nbsp;&nbsp;"/com/ibm/mashups/livetext/livetextchanged" <p />
   
   &nbsp;&nbsp;&nbsp;"/com/ibm/mashups/livetext/livetextcontentchanged" <p />

   Once an event is published to that topic, the implementation of this API should start using the given DOM node and processing all the
   decendant DOM nodes of that given node. That given node itself will not be processed. Event livetextchanged will cause the passed node and
   all its children to be parsed. Event livetextcontentchanged will cause only the passed node children to be parsed. <p /> 
   
   If a component wishes for a piece of markup fragment being processed, that component should publish an event to that topic, that event
   should also come with the DOM node which holds that piece of the markup fragment, callback functions and a flag to indicate how to proceed
   when an exception is thrown. The DOM node, flag and the callback functions are referred to event data below.<p />
   
   The event data must be an array of objects in the following format.<p />

   &nbsp;&nbsp;&nbsp;[domNode, continueAfterException, preProcessCallbackFunc, postProcessCallbackFunc] <p/>
   
   The first object in the event data array is the root node of the HTML markup fragment. If that object is not a valid DOM node, then no action
   will be taken. The second object in the array is a flag to indicate if the parsing should continue when an exception is caught. The third and
   the last object in the array are two callback functions. The preProcessCallbackFunc will be invoked before the parsing process starts, the
   postProcessCallbackFunc will be invoked after the parsing process finished. If none callback function is provided by the array, then no function
   will be invoked. The default value for continueAfterException flag is false. <p/>  
	
   If the node needs to be processed and the both pre and post process callback exist, then the call should look like this.<br /> <br />
   &nbsp;&nbsp;&nbsp;dojo.publish("/com/ibm/mashups/livetext/livetextchanged", <br />
   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[aDOMNode, false, callbackFunc1, callbackFunc2]); <br /><br />
   
   If all the nodes on the page need to be processed, and no callback functions are needed, then the call should look like this.<br/><br/> 
   &nbsp;&nbsp;&nbsp;dojo.publish("/com/ibm/mashups/livetext/livetextchanged", <br />
   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[document, true]); <br /><br />
   
   If the node in the event data is not a valid node, then the event will be ignored. Before a given DOM node being processed,
   The preProcesCallbackFunc will be called. After the given DOM node has been processed, the postProcessCallbackFunc will be called.
   If preProcessCallbackFunc or postProcessCallbackFunc is not null, then the callback function is guaranteed to be invoked regardless
   the status of the tag parsing process. The given DOM node is considered as the root node of given HTML fragment. All its decendent 
   child DOM nodes will be parsed. To make the process effecient, components should avoid passing the entire Document as the given DOM
   node unless it is the very first time.<p />
   
   Tag service configuration. <p/>
   This section describes the specific implementation of the IBM Lotus Mashups livetext services implementation.  The tag matching 
   method is not defined, it is up to the tag service implementation to define their own match method and tag processing invoke sequence.<p />
   
   Tags can be nested, this tag service can be configured to process nested tags or leave the nested tags to the container to
   process. <p />
   
   This service is configurable by providing a configuration file. The configuration file should be located at the same location
   where the tag service API implementation class is. The configuration file is simply a JSON type of code. It defines tags that this service can process. The 
   following is an example of this file. <p />
      
   &nbsp;&nbsp;&nbsp;[<br/>			
   &nbsp;&nbsp;&nbsp;{"match":"*.iWidgetSkin[skin]","processEnclosedTags":true,'waitOnPreTag':true,<br/>
   &nbsp;&nbsp;&nbsp;&nbsp;"module":"tagservices", "path":"../../tagservices", "baseClass":"tagservices.skins"},<br/>		
   &nbsp;&nbsp;&nbsp;{"match":"*.mm_iWidget","processEnclosedTags":false,'waitOnPreTag':false,<br/>
   &nbsp;&nbsp;&nbsp;&nbsp;"module":"tagservices", "path":"../../tagservices", "baseClass":"tagservices.widgets"}<br/>
   &nbsp;&nbsp;&nbsp;] <p />
    
   Each entry in this file presents a type of tags that this service can process. The order of these entries in the configuration
   file is important because the service will process the tags in this order. <p />
   
   Each entry should have the following element. <p />
   "match":  The CSS3 format for tag attribute match value. The value of this element determines if a tag should be processed by the services.<br/>
   "processEnclosedTags":  The value of true or false to determine if the service should process the nested tag. The default value is false.<br />
   "waitOnPreTag":  The value of true or false to determine if the service should process this entry defined tags before the process of the previous 
   entry defined tags have been completed. The default value is true. <br />  
   "module":  The implementation JavaScript module. This is required. <br />
   "path":  The path to the JavaScript implementation of the tag processor. This should be a URI to the JavaScript resource. This is required. <br />
   "baseClass":  The class which implement the tag processor API. <br />    
         
   Livetext service also provide a way for other components to dynamically add or remove a configuration entry. The addition or removal of the entries
   will not be persisted. It will only affect the current session. <br />
   
   If an entry needs to be added, then a component should publish to this topic with
   an entry.<br>

	&nbsp;&nbsp;&nbsp;"/com/ibm/mashups/livetext/configentryadded" <p />
	         
   If an entry needs to be removed, then a component should publish to this topic with
   an entry.<br>

	&nbsp;&nbsp;&nbsp;"/com/ibm/mashups/livetext/configentryremoved" <p />

   @author <a href="mailto:litong01@us.ibm.com">Tong Li</a>                  
   @ibm-api
  */ 
dojo.declare("com.ibm.mashups.livetext.ServiceModel", null, 
/** @lends com.ibm.mashups.livetext.ServiceModel */
{
	/**
    * Initiate the tag service. This should be only called once after a page is completely loaded. 
    * Normally it should be added to the document onload method. This method should initialize the
    * tag service which includes load the tag configuration and start parsing tags for the entire page.
    * @type void
 	*/ 
	init: function() {
	},

	/**
	* Tag changed event handler.  This handler will parse the passed in tag and all its children to be parsed.
 	* @param {DOMNode} domNode The DOM node whose decendent nodes to be parsed and processed. May be NULL
 	* @param {boolean} continueAfterException The flag to indicate if the service should continue the parsing and processing after an exception is caught.
 		If this parameter is missing, the default is false. May be NULL
 	* @param {com.ibm.mashups.livetext.callbackModel.preProcessCallbackFunc} preProcessCallbackFunc The callback function being called before a node being processed.May be NULL 
 	* @param {com.ibm.mashups.livetext.callbackModel.postProcessCallbackFunc} postProcessCallbackFunc The callback function being called after the a node being processed.May be NULL
 	* @type void
 	*/			
	onTagChanged: function(/* DOMNode */ domNode, 
						   /* boolean */ continueAfterException,
						   /* com.ibm.mashups.livetext.callbackModel.preProcessCallbackFunc */ preProcessCallbackFunc, 
						   /* com.ibm.mashups.livetext.callbackModel.postProcessCallbackFunc */ postProcessCallbackFunc) {
						   
	},
	
	/**
	* Tag content changed event handler. This handle will parsed all the passed in tag content to be parsed.
 	* @param {DOMNode} domNode The DOM node whose decendent nodes to be parsed and processed.May be NULL
 	* @param {boolean} continueAfterException The flag to indicate if the service should continue the parsing and processing after an exception is caught.
 		If this parameter is missing, the default is false.May be NULL
 	* @param {com.ibm.mashups.livetext.callbackModel.preProcessCallbackFunc} preProcessCallbackFunc The callback function being called before a node being processed.May be NULL 
 	* @param {com.ibm.mashups.livetext.callbackModel.postProcessCallbackFunc} postProcessCallbackFunc The callback function being called after the a node being processed.May be NULL
 	* @type void
 	*/			
	onTagContentChanged: function(/* DOMNode */ domNode, 
						   /* boolean */ continueAfterException,
						   /* com.ibm.mashups.livetext.callbackModel.preProcessCallbackFunc */ preProcessCallbackFunc, 
						   /* com.ibm.mashups.livetext.callbackModel.postProcessCallbackFunc */ postProcessCallbackFunc) {
						   
	},
	
	/**
	* The method allows component dynamically add a new configuration entry so that the service can start process the tags presented
	* by the entry.
	* @param {com.ibm.mashups.livetext.ConfigEntry} entry The entry that should be added to the configuration. Must not be NULL.
	* @type void
	*/
	onAddConfigEntry: function(/* com.ibm.mashups.livetext.ConfigEntry */ entry) {
	},
	
	/**
	* This method allows components dynamically remove a configuration entry so that the service will not process that type of tags which
	* are presented by the configuration entry.
	* @param {com.ibm.mashups.livetext.ConfigEntry} entry The entry that should be removed from the configuration. Must not be NULL.
	* @type void
	*/
	onRemoveConfigEntry: function(/* com.ibm.mashups.livetext.ConfigEntry */ entry) {
	}		
});


/**
  *This API defines the various callback functions for livetext services. These callback functions should be provided by
  *event publisher. <p />
   
  * @ibm-api
  * @author <a href="mailto:litong01@us.ibm.com">Tong Li</a>      
  * @class com.ibm.mashups.livetext.CallbackModel 
  * @name com.ibm.mashups.livetext.CallbackModel 
  */ 
dojo.declare("com.ibm.mashups.livetext.CallbackModel", null,
/** @lends com.ibm.mashups.livetext.CallbackModel */
{
	/**
	* The callback function which should be provided to livetext service by the event publisher. This
	* callback function provides a means by the event publisher and for the event publisher to run some code before 
	* the HTML fragment is processed. <p /> 
 	* @param {DOMNode} domNode The DOM node whose decendent nodes to be parsed and processed. May be NULL
 	* @param {Object[]} livetextList The list of the live text which will be processed. May be NULL
 	* @type void
 	*/			
	preProcessCallbackFunc: function(/* DOMNode */ domNode, /* Object[] */livetextList) {						   
	},
	/**
	* The callback function which should be provided to livetext service by the event publisher. This
	* callback function provides a means by the event publisher and for the event publisher to run some code 
	* after HTML fragment has been processed. <p /> 
 	* @param {DOMNode} domNode The DOM node whose decendent nodes to be parsed and processed. May be NULL
	* @param {Object[]} livetextList The list of the live text which will be processed. May be NULL
 	* @param {Object[]} errors The list of the exceptions occurred during the parsing and node handling process. If 
 	* there is no error, an empty array should be provided. May be NULL
 	* @type void
 	*/			
	postProcessCallbackFunc: function(/* DOMNode */ domNode, /* Object[] */livetextList, /* Object[] */ errors) {						   
	}		
});

/**
   This is the tag processor API. When a match tag is found by the livetext service according to the configuration, the service
   will hand the DOM node to the tag processor. Each tag processor must implement this interface. Each processor must also provide a 
   non-parameter constructor in addition to the processTag method. The non-parameter constructor is to ensure that the livetext service
   can create an instance of the implementation class without providing any more information. <p />    

   @ibm-api
   @author <a href="mailto:litong01@us.ibm.com">Tong Li</a>
   @class com.ibm.mashups.livetext.LivetextModel 
   @name com.ibm.mashups.livetext.LivetextModel 
  */ 
dojo.declare("com.ibm.mashups.livetext.LivetextModel", null, 
/** @lends com.ibm.mashups.livetext.LivetextModel */
{
	/**
	* The entry point of a tag processor. When a tag is found, the service will dynamically load the module which was designed to process
	* that type of tags, then create an instance of that tag module class if it has not been created, then call this method and hand the 
	* control to the tag module.
 	* @param {DOMNode} node The livetext DOM node to be processed. Must not be NULL.
 	* @type void
 	*/	
	processTag: function(/*DOMNode*/node) {
	}	
});

/**
   This API defines the exception which can be thrown by the semantic services during the parsing and processing
      
   An exceptionId should identify the current exception. It is a 7 character long string which uniquely identifies the 
   current abnormal condition of the parsing and processing. <p />   
   
   @ibm-api
   @author <a href="mailto:litong01@us.ibm.com">Tong Li</a>      
   @class com.ibm.mashups.livetext.Exception 
   @name com.ibm.mashups.livetext.Exception 
  */ 
dojo.declare("com.ibm.mashups.livetext.Exception", null,
/** @lends com.ibm.mashups.livetext.Exception */
{

	/**
    * Create new instance of livetext service exception. <p />
    *
    * @param {String} exceptionId The exception identifier. Must not be NULL.
 	* @param {Object} variable The list of the parameters to describe this error condition. May be NULL. This object must hold all the necessary 
 	* values to populate the exception message.  For example, a message may be defined as the following: <p />
 	*
 	* &nbsp;&nbsp;&nbsp;&nbsp;"The node ${id} does not have the necessary attribute ${attribute} or the attribute has unrecongnized values." <p/>
 	* When an exception is throw, the variable object should have members namded "id" and "attribute" respectively. Here is an example of such object <p />
 	*
 	* &nbsp;&nbsp;&nbsp;&nbsp;throw new com.ibm.mashups.livetext.exception('E000001', {'id':100, 'attribute':'DeJaWu'});
 	*
 	* @constructs 
	*/ 
	constructor: function(/* String */exceptionId, /* Object */ variable) {
	},
						   
	/**
	* Retrieve locale specific description by the using the status code. This method should not cause any exception. If 
	* no locale specific message is found, then an empty string will be returned. 
 	* @param {String[]} locales The list of the locales which the locale specific error should be looked up. May be NULL
 	* @returns {String} The locale specific exception message.
 	*/			
	getMessage: function(/* String[] */ locales) {					   
	}
});

/**
   This API defines the service model configuration entry
         
   @ibm-api
   @author <a href="mailto:litong01@us.ibm.com">Tong Li</a>      
   @class com.ibm.mashups.livetext.ConfigEntry 
   @name com.ibm.mashups.livetext.ConfigEntry 
  */ 
dojo.declare("com.ibm.mashups.livetext.ConfigEntry", null,
/** @lends com.ibm.mashups.livetext.ConfigEntry */
{

	/**
    * Create new instance of livetext config entry. <p />
    *
    * @param {String} match the CSS3 Selection expression to match a live text. Must not be NULL
 	* @param {Boolean} processEnclosedTags should all the nested live text be processed in this request. If true, then all the live text will be processed. If false,
 	* only the out most live text of this type will be processed. May be NULL
 	* @param {String} module the module to register the implementation livetextModel under. Must not be NULL.
 	* @param {String} path the url to load the livetextModel implementation resource file. Must not be NULL.
 	* @param {String} baseClass the JavaScript class which implement the livetextModel. Must not be NULL.<p/> 
 	*
	*/ 
	constructor: function(/* String */ match, /* Boolean */ processEnclosedTags, /* String */module, 
						  /* String */ path, /* String */ baseClass) {
	},
						   
	/**
    * Compare if this configuration entry is same as the given entry. 
    *
    * @param {com.ibm.mashups.livetext.ConfigEntry} entry The entry to be compared with this entry. Must not be NULL.
 	* @returns {Boolean} The value indicats if the two entries are the same. 
	*/ 						   
	isEqual: function(/* ConfigEntry */ entry) {
	}					   						   
});
